using Godot;
using System;

public class InteractionModes : Node
{
    [Export]
    private NodePath _buildModePath;
    private BuildMode _buildMode;
    [Export]
    private NodePath _selectionModePath;
    private SelectionMode _selectionMode;

    [Signal]
    delegate void OnInteractionModeChanged(Mode oldMode, Mode newMode);

    private Mode _current = null;

    public override void _Ready()
    {
        base._Ready();

        _selectionMode = GetNode<SelectionMode>(_selectionModePath);
        _buildMode = GetNode<BuildMode>(_buildModePath);
    }

    public override void _Process(float delta)
    {
        base._Process(delta);

        if (_current == null)
        {
            Reset();
        }
    }

    public void _on_Build_Mode_OnModeEntered()
    {
        _selectionMode.Disable();
        ChangeMode(_selectionMode, _buildMode);
    }
    public void _on_Build_Mode_OnModeExited()
    {
    }

    public void _on_Selection_Mode_OnModeEntered()
    {
        _buildMode.Disable();
        ChangeMode(_buildMode, _selectionMode);
    }
    public void _on_Selection_Mode_OnModeExited()
    {
    }

    private void ChangeMode(Mode oldMode, Mode newMode)
    {
        if (oldMode == newMode)
        {
            throw new InvalidOperationException();
        }

        if (_current != null && oldMode != _current)
        {
            throw new InvalidOperationException();
        }

        _current = newMode;
        EmitSignal(nameof(OnInteractionModeChanged), oldMode, newMode);
    }

    public void Reset()
    {
        _buildMode.Disable();
        _selectionMode.Enable();
    }
}
